#include <FastLED.h>

#define LED_PIN     9       // Pin where the LED strip is connected
#define LED_PIN1    10      // Pin where the second LED strip is connected
#define LED_PIN2    12      // Pin where the third LED strip is connected
#define buttonPin   11      // Pin where the button is connected
#define NUM_LEDS    330     // Number of LEDs in the strip
#define BRIGHTNESS  255     // Maximum brightness
#define LED_TYPE    WS2812B // Type of LED strip
#define COLOR_ORDER GRB     // Color order for the strip
#define DEBOUNCE_DELAY 200   // Minimum time between button presses (ms)

volatile int pattern = 0;      // Current pattern number
volatile bool buttonPressed = false; // Flag to indicate button press
unsigned long lastPressTime = 0;     // Last time the button was pressed

CRGB leds[NUM_LEDS];
CRGB leds1[NUM_LEDS];
CRGB leds2[NUM_LEDS];

void setup() {
    FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
    FastLED.addLeds<LED_TYPE, LED_PIN1, COLOR_ORDER>(leds1, NUM_LEDS).setCorrection(TypicalLEDStrip);
    FastLED.addLeds<LED_TYPE, LED_PIN2, COLOR_ORDER>(leds2, NUM_LEDS).setCorrection(TypicalLEDStrip);
    FastLED.setBrightness(BRIGHTNESS);

    pinMode(buttonPin, INPUT_PULLUP);  // Use INPUT_PULLUP for stable signal
    attachInterrupt(digitalPinToInterrupt(buttonPin), changePattern, FALLING);  // Trigger on button press

    Serial.begin(9600);
}

void changePattern() {
    unsigned long currentTime = millis();
    // Check if the button press is within the debounce time
    if (currentTime - lastPressTime > DEBOUNCE_DELAY) {
        pattern++;
        if (pattern > 5) pattern = 0;  // Cycle through patterns 1 to 5
        buttonPressed = true;
        lastPressTime = currentTime;  // Update the last press time
        Serial.print("Pattern: ");
        Serial.println(pattern);
    }
}

void loop() {
    if (buttonPressed) {
        buttonPressed = false;  // Reset the flag after processing the button press
    }

    // Pattern 0: All LEDs off
    if (pattern == 0) {
        fill_solid(leds, NUM_LEDS, CRGB::Black);
        fill_solid(leds1, NUM_LEDS, CRGB::Black);
        fill_solid(leds2, NUM_LEDS, CRGB::Black);
        FastLED.show();
        delay(10);
    }

    // Pattern 1: Solid Red
    if (pattern == 1) {
        fill_solid(leds, NUM_LEDS, CRGB::Red);
        fill_solid(leds1, NUM_LEDS, CRGB::Red);
        fill_solid(leds2, NUM_LEDS, CRGB::Red);
        FastLED.show();
        delay(10);
    }

    // Pattern 2: Blinking Red
    if (pattern == 2) {
        fill_solid(leds, NUM_LEDS, CRGB::Red);
        fill_solid(leds1, NUM_LEDS, CRGB::Red);
        fill_solid(leds2, NUM_LEDS, CRGB::Red);
        FastLED.show();
        delay(100);
        fill_solid(leds, NUM_LEDS, CRGB::Black);
        fill_solid(leds1, NUM_LEDS, CRGB::Black);
        fill_solid(leds2, NUM_LEDS, CRGB::Black);
        FastLED.show();
        delay(100);
    }

    // Pattern 3: Blue and White Chase
    if (pattern == 3) {
        for (int i = 0; i < NUM_LEDS; i++) {
            leds[i] = CRGB::Blue;
            leds1[i] = CRGB::Blue;
            leds2[i] = CRGB::Blue;
            FastLED.show();
            delay(1);
        }
        for (int i = NUM_LEDS - 1; i >= 0; i--) {
            leds[i] = CRGB::White;
            leds1[i] = CRGB::White;
            leds2[i] = CRGB::White;
            FastLED.show();
            delay(1);
        }
        for (int i = 0; i < NUM_LEDS; i++) {
            leds[i] = CRGB::Red;
            leds1[i] = CRGB::Red;
            leds2[i] = CRGB::Red;
            FastLED.show();
            delay(1);
        }
        for (int i = NUM_LEDS - 1; i >= 0; i--) {
            leds[i] = CRGB::Green;
            leds1[i] = CRGB::Green;
            leds2[i] = CRGB::Green;
            FastLED.show();
            delay(1);
        }
    }

    // Pattern 4: Pulse Colors
    if (pattern == 4) {
        pulseColor(CRGB::OrangeRed);
        pulseColor(CRGB::Yellow);
        pulseColor(CRGB::Green);
        pulseColor(CRGB::Aqua);
        pulseColor(CRGB::Blue);
        pulseColor(CRGB::Indigo);
        pulseColor(CRGB::DeepPink);
    }

    // Pattern 5: Alternating Red, Green, Blue
    if (pattern == 5) {
        for (int i = 0; i < NUM_LEDS; i++) {
            if (i % 3 == 0) {
                leds[i] = CRGB::Red;
                leds1[i] = CRGB::Red;
                leds2[i] = CRGB::Red;
            } else if (i % 3 == 1) {
                leds[i] = CRGB::Green;
                leds1[i] = CRGB::Green;
                leds2[i] = CRGB::Green;
            } else {
                leds[i] = CRGB::Blue;
                leds1[i] = CRGB::Blue;
                leds2[i] = CRGB::Blue;
            }
        }
        FastLED.show();
        delay(100);
    }
}

void pulseColor(CRGB color) {
    for (int brightness = 0; brightness <= 255; brightness++) {
        applyBrightness(color, brightness);
        FastLED.show();
        delay(5);
    }
    for (int brightness = 255; brightness >= 0; brightness--) {
        applyBrightness(color, brightness);
        FastLED.show();
        delay(5);
    }
}

void applyBrightness(CRGB color, uint8_t brightness) {
    for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = color;
        leds1[i] = color;
        leds2[i] = color;
        leds[i].fadeLightBy(255 - brightness);
        leds1[i].fadeLightBy(255 - brightness);
        leds2[i].fadeLightBy(255 - brightness);
        if (brightness == 0) {
            leds[i] = CRGB::Black;
            leds1[i] = CRGB::Black;
            leds2[i] = CRGB::Black;
        }
    }
}
